@@ -1,5 +1,5 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "compiler-explorer" "0.8.0"
+(define-package "compiler-explorer" "20250928.1426"
   "Compiler explorer client (godbolt.org)."
   '((emacs "28.1")
     (plz   "0.9")
@@ -7,8 +7,8 @@
     (map   "3.3.1")
     (seq   "2.23"))
   :url "https://github.com/mkcms/compiler-explorer.el"
-  :commit "b2ea96c2989a953aca02ccfc966877fb6c538521"
-  :revdesc "v0.8.0-0-gb2ea96c2989a"
+  :commit "9e32e10b13a53b1fbf5471c34e4c0ea7a5db9057"
+  :revdesc "9e32e10b13a5"
   :keywords '("c" "tools")
   :authors '(("Michał Krzywkowski" . "k.michal@zoho.com"))
   :maintainers '(("Michał Krzywkowski" . "k.michal@zoho.com")))
