@@ -4,8 +4,8 @@
 
 ;; Author: Michał Krzywkowski <k.michal@zoho.com>
 ;; Keywords: c, tools
-;; Package-Version: 0.8.0
-;; Package-Revision: v0.8.0-0-gb2ea96c2989a
+;; Package-Version: 20250928.1426
+;; Package-Revision: 9e32e10b13a5
 ;; Homepage: https://github.com/mkcms/compiler-explorer.el
 ;; Package-Requires: ((emacs "28.1")
 ;;                    (plz "0.9")
@@ -31,7 +31,7 @@
 ;;
 ;;; compiler-explorer.el
 ;;
-;; Package that provides a client for https://godbolt.org service.
+;; Package that provides a simple client for https://godbolt.org service.
 ;;
 ;;
 ;;; Usage
@@ -42,9 +42,6 @@
 ;; compiled asm code.  Another buffer displays output of the compiled and
 ;; executed program.
 ;;
-;;
-;;; Compilation
-;;
 ;; M-x `compiler-explorer-set-compiler' changes the compiler for current
 ;; session.
 ;;
@@ -54,41 +51,15 @@
 ;; it to current compilation.  M-x `compiler-explorer-remove-library'
 ;; removes them.
 ;;
-;;
-;;; Execution
-;;
 ;; M-x `compiler-explorer-set-execution-args' sets the arguments for the
 ;; executed program.
 ;;
 ;; M-x `compiler-explorer-set-input' reads a string from minibuffer that
 ;; will be used as input for the executed program.
 ;;
-;;
-;;; Session management
-;;
-;; M-x `compiler-explorer-new-session' kills the current session and
-;; creates a new one, asking for source language.
-;;
-;; M-x `compiler-explorer-previous-session' lets you restore previous
-;; sessions.
-;;
-;; M-x `compiler-explorer-discard-session' kills the current or selected
-;; sessions and forgets about them forever.
-;;
-;; M-x `compiler-explorer-exit' kills the current session.
-;;
-;;
-;;; ASM
-;;
-;; M-x `compiler-explorer-browse-opcode-documentation' opens a website
-;; that contains the documentation for the opcode at point.
-;;
 ;; M-x `compiler-explorer-jump' jumps to ASM block for the source line at
 ;; point and vice versa.
 ;;
-;;
-;;; Tools
-;;
 ;; M-x `compiler-explorer-add-tool' asks for the name of a tool, adds it
 ;; to current compilation and displays a new buffer showing the tool's
 ;; output.
@@ -102,13 +73,18 @@
 ;; M-x `compiler-explorer-set-tool-input' reads a string from minibuffer
 ;; that will be used as input for an added tool.
 ;;
-;;
-;;
-;;; Other commands
-;;
 ;; M-x `compiler-explorer-load-example' prompts for a name of a builtin
 ;; example and loads it.
 ;;
+;; M-x `compiler-explorer-new-session' kills the current session and
+;; creates a new one, asking for source language.
+;;
+;; M-x `compiler-explorer-previous-session' lets you restore previous
+;; sessions.
+;;
+;; M-x `compiler-explorer-discard-session' kills the current or selected
+;; sessions and forgets about them forever.
+;;
 ;; M-x `compiler-explorer-make-link' generates a link for current
 ;; compilation so it can be opened in a browser and shared.
 ;;
@@ -117,37 +93,7 @@
 ;;
 ;; M-x `compiler-explorer-layout' cycles between different layouts.
 ;;
-;; ## Customization
-;;
-;; Additional customization is possible via M-x `customize-group'
-;; `compiler-explorer'.
-;;
-;; ### Usage with language servers
-;;
-;; The following snippet sets up the built-in eglot package to start a
-;; language server for each session, and to automatically create and
-;; update a `compile_flags.txt' file (recognized by clangd) to have the
-;; same compiler arguments that are set for the current session.  This
-;; requires that the `compiler-explorer-make-temp-file' custom variable is
-;; non-nil.
-;;
-;;
-;;   (add-hook 'compiler-explorer-new-session-hook #'eglot-ensure)
-;;
-;;   (defun my/compiler-explorer-params-change-hook (what value)
-;;     "Hook run when compilation parameters WHAT change to VALUE."
-;;     (pcase what
-;;       ('compiler-args
-;;        (with-current-buffer compiler-explorer--buffer
-;;          (when (derived-mode-p 'c-mode 'c++-mode)
-;;            (with-temp-file "compile_flags.txt"
-;;              (insert (mapconcat #'identity
-;;                                 (split-string-and-unquote value) "\n")))
-;;            (when (eglot-current-server)
-;;              (eglot-reconnect (eglot-current-server))))))))
-;;
-;;   (add-hook 'compiler-explorer-params-change-hook
-;;             #'my/compiler-explorer-params-change-hook)
+;; M-x `compiler-explorer-exit' kills the current session.
 ;;
 ;;
 
@@ -269,23 +215,15 @@ URL parameters: ?FIELD1=VALUE1&FIELD2=VALUE2..."
 Keys are strings of the form \\='ISET:OPCODE\\=', where ISET is the
 compiler's :instructionSet.
 
-Values are plists, which contain the documentation for the opcode.
-Values can also be t, which means no documentation is available for the
-key.")
+Values are strings, which contain the documentation for the
+opcode.  Values can also be t, which means no documentation is
+available for the key.")
 
-(defun ce--asm-opcode-doc (instruction-set opcode callback &optional sync)
+(defun ce--asm-opcode-doc (instruction-set opcode callback)
   "Get documentation for OPCODE in INSTRUCTION-SET and call CALLBACK.
 Opcode should be a string.  INSTRUCTION-SET should be a valid
 :instructionSet of some compiler.
 
-CALLBACK is called with a plist argument.  The plist is of the
-form: (:tooltip TOOLTIP :html HTML :url URL).  TOOLTIP is the short
-documentation string.  HTML is full documentation in HTML.  URL is the
-link to get more information for the opcode.
-
-If SYNC is true, then wait until response arrives and call CALLBACK
-immediately.
-
 The return value will be:
 - nil if documentation is not available
 - t if it is (and callback was called)
@@ -293,29 +231,20 @@ The return value will be:
 sent, but the documentation is not available yet."
   (let* ((key (format "%s:%s" instruction-set opcode))
          (cache ce--asm-opcode-docs-cache)
-         (resp (gethash key cache))
-         (handler (lambda (response)
-                    (funcall callback response)
-                    (puthash key response cache))))
+         (resp (gethash key cache)))
     (cond
-     ((and resp (listp resp)) (funcall callback resp) t)
+     ((stringp resp) (funcall callback resp) t)
      ((eq resp t) nil)
      (t
-      (setq resp
-            (condition-case err
-                (plz 'get
-                  (ce--url "asm" instruction-set opcode)
-                  :headers '(("Accept" . "application/json"))
-                  :then (if sync 'sync handler)
-                  :else (lambda (_err) (puthash key t cache))
-                  :as #'ce--parse-json)
-              (plz-error
-               (puthash key t cache)
-               (or sync (signal (car err) (cdr err)))
-               nil)))
-      (if sync
-          (and resp (funcall handler resp) t)
-        'async)))))
+      (plz 'get
+        (ce--url "asm" instruction-set opcode)
+        :headers '(("Accept" . "application/json"))
+        :then (pcase-lambda ((map :tooltip))
+                (funcall callback tooltip)
+                (puthash key tooltip cache))
+        :else (lambda (_err) (puthash key t cache))
+        :as #'ce--parse-json)
+      'async))))
 
 (defvar ce--examples nil)
 (defun ce--examples (&optional lang)
@@ -1369,11 +1298,11 @@ This is eldoc function for compiler explorer."
     (ce--asm-opcode-doc
      (plist-get ce--compiler-data :instructionSet)
      opcode
-     (pcase-lambda ((map :tooltip))
+     (lambda (doc)
        (funcall callback
                 (with-temp-buffer
                   (text-mode)
-                  (insert tooltip)
+                  (insert doc)
                   (fill-paragraph)
                   (buffer-string))
                 :thing opcode)))))
@@ -1474,17 +1403,7 @@ It must have been created with `compiler-explorer--current-session'."
                                                       (consp obj)
                                                       (vectorp obj))))))
       (unless (funcall pred val)
-        (error "Invalid %S: %s" sym val)))
-    (dolist (elt libs)
-      (unless (and (consp elt)
-                   (stringp (car elt))
-                   (stringp (cdr elt)))
-        (error "Invalid library: %s" elt)))
-    (dolist (elt tools)
-      (unless (and (listp elt)
-                   (= 3 (length elt))
-                   (cl-every #'stringp elt))
-        (error "Invalid tool: %s" elt)))
+        (error "Invalid %s: %s" sym val)))
 
     (let ((ce--inhibit-request t))
       (ce-new-session lang-name compiler)
@@ -1509,7 +1428,6 @@ It must have been created with `compiler-explorer--current-session'."
 
 (defun ce--save-sessions ()
   "Save all sessions to a file."
-  (remove-hook 'kill-emacs-hook #'ce--save-sessions)
   (let ((current-session (and (ce--active-p) (ce--current-session))))
     (when current-session
       (push current-session ce--session-ring))
@@ -1544,10 +1462,7 @@ It must have been created with `compiler-explorer--current-session'."
   ;; Set the header line status to "Wait..."
   (force-mode-line-update t))
 
-(defvar ce-mode-map
-  (let ((map (make-sparse-keymap)))
-    (define-key map [remap display-local-help] #'eldoc-doc-buffer)
-    map)
+(defvar ce-mode-map (make-sparse-keymap)
   "Keymap used in all compiler explorer buffers.")
 
 (define-minor-mode ce--local-mode
@@ -1595,8 +1510,6 @@ It must have been created with `compiler-explorer--current-session'."
   ce--local-mode-maybe-enable
   :lighter " CE"
   :keymap ce-mode-map
-  :require 'compiler-explorer
-  (add-hook 'kill-emacs-hook #'ce--save-sessions)
   (unless ce-mode
     (ce--cleanup)))
 
@@ -1682,28 +1595,6 @@ the same source line."
                  (length group)))
     (error "No corresponding ASM or source code block at point")))
 
-(defun ce-browse-opcode-documentation (opcode)
-  "Browse documentation for OPCODE in external browser.
-This looks up the URL for the specific opcode docs in the
-compiler-explore API and navigates to it.  Interactively, opcode is the
-symbol at point.  With a prefix argument, a symbol is read from the
-minibuffer."
-  (interactive
-   (and (or (ce--active-p) (user-error "Not in a `compiler-explorer' session"))
-        (list
-         (if current-prefix-arg
-             (read-from-minibuffer "Show opcode documentation: " )
-           (or (thing-at-point 'symbol)
-               (user-error "There is no symbol at point"))))))
-  (unless (ce--active-p)
-    (error "Not in a `compiler-explorer' session"))
-  (unless (ce--asm-opcode-doc
-           (plist-get ce--compiler-data :instructionSet)
-           opcode
-           (pcase-lambda ((map :url)) (browse-url-xdg-open url))
-           'sync)
-    (error "No documentation for %s" opcode)))
-
 (defun ce-set-input (input)
   "Set the input to use as stdin for execution to INPUT, a string."
   (interactive (list (if (ce--active-p)
@@ -2024,17 +1915,15 @@ all the previous sessions one by one."
   (unless nth
     (setq nth 0))
   (let ((prev (nth nth ce--session-ring))
-        (current (and (ce--session-savable-p) (ce--current-session)))
-        (success nil))
+        (current (and (ce--session-savable-p) (ce--current-session))))
     (setq ce--session-ring (seq-remove-at-position ce--session-ring nth))
 
-    (condition-case err
+    (condition-case nil
         (prog1 t
           (unwind-protect
               (let ((ce--session-ring nil))
                 ;; Override the ring to not mess with it.
-                (ce--restore-session prev)
-                (setq success t))
+                (ce--restore-session prev))
             ;; Insert last session into the ring as the *oldest* item.  We have
             ;; to do this, otherwise we would only be able to cycle between two
             ;; sessions.
@@ -2043,14 +1932,11 @@ all the previous sessions one by one."
 
             ;; Redefine the menu with the ring updated (for "Restore session"
             ;; submenu)
-            (when success
-              (ce--define-menu))))
+            (ce--define-menu)))
       (error
        (ce--cleanup 'skip-save-session)
        (display-warning
-        'compiler-explorer
-        (format "Previous session is corrupt: %s" (error-message-string err))
-        :warning)
+        'compiler-explorer "Previous session appears to be corrupt" :warning)
        nil))))
 
 (defun ce-discard-session (&optional indices interactive)
@@ -2475,6 +2361,8 @@ The hook `compiler-explorer-hook' is always run at the end."
      (t (call-interactively #'ce-new-session))))
   (run-hooks 'ce-hook))
 
+(add-hook 'kill-emacs-hook #'ce--save-sessions)
+
 (provide 'compiler-explorer)
 ;;; compiler-explorer.el ends here
 
